<?php

namespace App\Service;

class RappiMenu {

    private $rappi;

    public function __construct(\App\Integration\Rappi $rappi) {
        $this->rappi = $rappi;
    }

    function updateMenu($token, $params) {
        $menuStatus = $this->rappi->getMenuStatus($token, $params['merchantId']);
        if($menuStatus['status'] != 'Disponível') {
            throw new \Exception("O cardápio não está disponível para alterações");
        }

        $currentMenu = $this->rappi->getMenu($token,$params['merchantId']);
        foreach ($params['items'] as $modifiedItem) {
            if ($modifiedItem['type'] === 'C') {
                $parentIndex = $modifiedItem['parentIndex'];
                $arrayIndex = $modifiedItem['arrayIndex'];

                $currentMenu['items'][$parentIndex]['children'][$arrayIndex]['sku'] = $modifiedItem['finalSku'];
                $currentMenu['items'][$parentIndex]['children'][$arrayIndex]['price'] = $modifiedItem['finalPrice'];
            } else {
                $arrayIndex = $modifiedItem['arrayIndex'];

                $currentMenu['items'][$arrayIndex]['sku'] = $modifiedItem['finalSku'];
                $currentMenu['items'][$arrayIndex]['price'] = $modifiedItem['finalPrice'];
            }
        }
        $response = $this->rappi->postMenu($token, $currentMenu);

        return $response;
    }

    private function validateItemSimple(&$item) {
       if ($item['price'] < 0.01 || strstr($item['sku'],"|")) {
            $item['hasInvalidOptions'] = true;
       }
    }

    private function validateCombo(&$item) {
        foreach ($item['children'] as &$option) {
            if (!$option['price'] > 0 || empty($option['sku']) || strstr($option['sku'], "|")) {
                $item['hasInvalidOptions'] = true;
                return;
            }
        }
    }

    private function validateItemWithAddOns(&$item) {
        foreach ($item['children'] as $option) {
            $isValidOption = (empty($option['price']) && (strlen($option['sku']) === 5 && substr($option['sku'], 2, 1) === '|') ||
             ($option['price'] > 0.01 && !empty($option['sku'])));

            if (!$isValidOption) {
                $item['hasInvalidOptions'] = true;
                return;
            }
        }
    }

    private function itemValidations(&$item) {
        $item['hasInvalidOptions'] = false;

        if (is_null($item['sku']) || $item['sku'] === '' || $item['type'] != 'PRODUCT') {
            $item['hasInvalidOptions'] = true;
            return;
        }

        if(empty($item['children'])) {
            $this->validateItemSimple($item);
            return;
        }

        if (empty($item['price'])) {
            $this->validateCombo($item);
            return;
        }

        if($item['price'] >= 0.01) {
            $this->validateItemWithAddOns($item);
            return;
        }
    }

    function formatData(&$apiResponse) {
        $formattedData = [];
        $idx = 0;

        foreach ($apiResponse['items'] as $index => $item) {
            $idx++;
            $item['uniqueKey'] = !empty($item['rappiIds']) ?
                                 implode('_', $item['rappiIds']) : $idx;
            $this->itemValidations($item);
            $categoryId = $item['category']['id'];
            $categoryName = $item['category']['name'];
            if (!isset($formattedData[$categoryId])) {
                $formattedData[$categoryId] = [
                    'id' => $categoryId,
                    'name' => $categoryName,
                    'items' => [],
                ];
            }

            $isValid = !$item['hasInvalidOptions'];

            $formattedData[$categoryId]['items'][] = [
                'id'            => $item['uniqueKey'],
                'sku'           => $item['sku'] ?? '',
                'name'          => $item['name'],
                'price'         => $item['price'],
                'combo'         => $item['combo'],
                'isValid'       => $isValid,
                "type"          => "I",
                "arrayIndex"    => $index,
            ];

            $lastParentId = $item['uniqueKey'];

            if (!empty($item['children'])) {
                $optionGroup = [];

                foreach ($item['children'] as $optionIndex => $option) {
                    $idx++;
                    $option['uniqueKey'] = is_array($option['rappiIds']) && !empty($option['rappiIds']) ?
                                           implode('_', $option['rappiIds']) : $idx;

                    if (isset($option['category']) && !empty($option['category']) && !in_array($option['category']['id'], $optionGroup)) {
                        $formattedData[$categoryId]['items'][] = [
                            'parentId'  => $lastParentId,
                            'id'        => $option['category']['id'],
                            'name'      => $option['category']['name'],
                            'isValid'   => $isValid,
                        ];

                        $optionGroup[] = $option['category']['id'];
                    }

                    $formattedData[$categoryId]['items'][] = [
                        'parentId'      => $option['category']['id'] ?? $lastParentId,
                        'id'            => $option['uniqueKey'],
                        'sku'           => $option['sku'],
                        'name'          => $option['name'],
                        'price'         => $option['price'],
                        'isValid'       => $isValid,
                        "type"          => "C",
                        "arrayIndex"    => $optionIndex,
                        "parentIndex"   => $index,
                    ];
                }
            }
        }
        return array_values($formattedData);
    }
}
